/***************************************************************************
                          shmap.h  -  description
                             -------------------
    begin                : Wed Feb 6 2002
    copyright            : (C) 2002 by Margus Laak
    email                : margus@mail.ee
 ***************************************************************************/

/***************************************************************************
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA *
 ***************************************************************************/



#ifndef _SHMAP_I
#define _SHMAP_I

#include <stdlib.h>
#include <shmap_config.h>

/*
        function module definition
*/
typedef struct shmap_module shmap_module;
typedef struct shmap_opts shmap_opts;
typedef struct shmap shmap;
typedef struct _shmap_value _shmap_value;
typedef struct shmap_perm shmap_perm;

/*
        well, currently we define all the module types
        over here
*/
#define SHMAP_TYPE_LL    1
#define SHMAP_TYPE_HM    2
#define SHMAP_TYPE_BTREE 3
#define SHMAP_TYPE_FIFO  4

// define the default type
#define SHMAP_TYPE_DEFAULT SHMAP_TYPE_LL

// decleare the cleanup function type
typedef void (*shmap_cleanup)(_shmap_value *, int *);
// the talkback values can be
#define SHMAP_CLEANUP_OK          0xffffffff // free all esources
#define SHMAP_CLEANUP_NONE        0x00000000 // free only the strycture
#define SHMAP_CLEANUP_FREE_KEY    0x00000001 // free key value
#define SHMAP_CLEANUP_FREE_VALUE  0x00000002 // free value value


struct shmap_opts {
        void * p_mem;      // the memory segment
        size_t size;       // the maximum size of map
        char   type;       // type of hashmap
        int    flags;      // flags for that hashmap
        char * p_mem_name; // the shared memory segment name (for client/server programs)
        };

struct shmap {
        void *toplevel;

        void * mem_segm;
        void * locks;

        unsigned int items;
        unsigned int attached; // the count of attached processes to this map

        // the position of functions in __shmap_modules array
        int    func_pos;

        shmap_opts   * opts;  // options that this map was initialized with

        // some algorithms use data seeking cache for quicker access
        void * cache;
};

// shmap permissions
struct shmap_perm {
        int mode;
        int uid;
        int gid;
};

typedef enum{
        SHMAP_LOCK_READ,
        SHMAP_LOCK_WRITE
        } shmap_lock_mode;

typedef struct _shmap_lock shmap_lock;

/****************************************
 *
 *        the API functions itself
 *
 ****************************************/
shmap      * shmap_init ( shmap_opts * p_opts );
void       * shmap_get_value( shmap *, const char *, int );
int          shmap_add_value ( shmap *, const char *, const char *);
int          shmap_del_value ( shmap *, const char *);
char       * shmap_get_sub ( shmap *, const char *, int, int, int);
int          shmap_del_sub ( shmap *, const char *, int);
int          shmap_clear ( shmap * );
int          shmap_dispose ( shmap * );
size_t       shmap_items ( shmap * );
void       * shmap_memsegm ( shmap * );
void         shmap_dump ( shmap *, int );
size_t       shmap_available ( shmap * );
shmap_lock * shmap_lock_init ( void * p_mem, const char * path );
shmap_lock * shmap_lock_get ( shmap * map );
int          shmap_lock_set ( shmap * map, shmap_lock * lock );
int          shmap_lock_del ( shmap * map );
int          shmap_lock_dispose ( void * p_mem, shmap_lock * lock );
int          shmap_dolock ( shmap_lock * lock, shmap_lock_mode mode );
int          shmap_unlock ( shmap_lock * lock );

/*
        shmap_opts_* family functions
*/
shmap_opts * shmap_opts_init ( void );
void         shmap_opts_setmem ( shmap_opts * p_opts, void * p_mem );
void         shmap_opts_settype ( shmap_opts * p_opts, char map_type );
void         shmap_opts_setsize ( shmap_opts * p_opts, size_t size );
void         shmap_opts_setflag ( shmap_opts * p_opts, int option );
void         shmap_opts_unsetflag ( shmap_opts * p_opts, int option );
void         shmap_opts_setname ( shmap_opts * p_opts, char * filename );

extern shmap_opts __def_opts;
#define SHMAP_DEFAULT &__def_opts

/****************************************
        aliases to API functions
 ****************************************/
#define shmap_add(m,k,v) shmap_add_value(m,k,v)
#define shmap_get(m,k) (char*)shmap_get_value(m,k,1)
#define shmap_del(m,k) shmap_del_value(m,k)

#endif
